@@ -2,14 +2,13 @@
 #
 # This file is part of Invenio.
 # Copyright (C) 2016-2018 CERN.
+# Copyright (C)      2022 TU Wien.
 #
 # Invenio is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
 
 """Proxy to the current stats module."""
 
-from __future__ import absolute_import, print_function
-
 from kombu import Exchange
 
 from .utils import default_permission_factory
@@ -66,8 +65,8 @@ access control mechanisms.
 
 
 STATS_MQ_EXCHANGE = Exchange(
-    'events',
-    type='direct',
-    delivery_mode='transient',  # in-memory queue
+    "events",
+    type="direct",
+    delivery_mode="transient",  # in-memory queue
 )
 """Default exchange used for the message queues."""
