@@ -2,14 +2,13 @@
 #
 # This file is part of Invenio.
 # Copyright (C) 2017-2018 CERN.
+# Copyright (C)      2022 TU Wien.
 #
 # Invenio is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
 
 """Errors used in Invenio-Stats."""
 
-from __future__ import absolute_import, print_function
-
 from invenio_rest.errors import RESTException
 
 ##
@@ -36,6 +35,7 @@ class DuplicateAggregationError(Exception):
 class DuplicateQueryError(Exception):
     """Error raised when a duplicate aggregation is detected."""
 
+
 ##
 #  Aggregation errors
 ##
@@ -49,6 +49,7 @@ class NotSupportedInterval(Exception):
 #  Query errors
 ##
 
+
 class InvalidRequestInputError(RESTException):
     """Error raised when the request input is invalid."""
 
@@ -70,5 +71,6 @@ class UnknownQueryError(RESTException):
         """
         super(RESTException, self).__init__()
         self.query_name = query_name
-        self.description = 'Unknown statistic \'{}\''.format(query_name)
+        self.description = "Unknown statistic '{}'".format(query_name)
+
     code = 400
